/**
 * AutoDAO - Generic DAO on steroids implementation for Java.
 * Copyright (C) 2008-2009  Marat Radchenko <slonopotamus@users.sourceforge.net>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.autodao;

import java.lang.annotation.Annotation;

/**
 * Pluggable mechanism for query argument transformation.
 * <p/>
 * In order to define custom transformer, add a bean with "autodaoQueryArgumentTransformer" name to Spring context.
 * <p/>
 * If no such bean specified, AutoDAO uses {@link DefaultQueryArgumentTransformer}.
 */
public interface QueryArgumentTransformer {
  /**
   * Performs query argument transformation.
   * @param declaredType argument type declared in finder method signature.
   * @param annotations  annotations on method parameter.
   * @param argument     @return transformed argument that'll be passed to underlying ORM.
   * @return transformed query argument that'll be passed to Hibernate.
   */
  Object transform(Class<?> declaredType, Annotation[] annotations, Object argument);
}
