/**
 * AutoDAO - Generic DAO on steroids implementation for Java.
 * Copyright (C) 2008-2009  Marat Radchenko <slonopotamus@users.sourceforge.net>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.autodao.hibernate;

import org.springframework.beans.factory.support.BeanDefinitionBuilder;
import org.springframework.beans.factory.xml.AbstractSingleBeanDefinitionParser;
import org.springframework.beans.factory.xml.ParserContext;
import org.springframework.beans.factory.xml.XmlReaderContext;
import org.springframework.util.StringUtils;
import org.w3c.dom.Element;

/**
 * Processes <code>dao:hibernate</code> elements and builds Spring bean.
 * <p/>
 * Creates {@link HibernateDao} bean and wrap it with {@link HibernateDaoProxyFactoryBean} so that AutoDAO magic works.
 * <p/>
 * NOT FOR PUBLIC USE.
 */
public final class HibernateDaoBeanDefinitionParser extends AbstractSingleBeanDefinitionParser {

  /** {@inheritDoc} */
  @Override
  protected boolean shouldGenerateIdAsFallback() {
    return true;
  }

  /** {@inheritDoc} */
  @Override
  protected void doParse(final Element element, final ParserContext context, final BeanDefinitionBuilder builder) {
    final String daoInterfaceName = element.getAttribute("interface");
    final XmlReaderContext xmlReaderContext = context.getReaderContext();
    if (!StringUtils.hasText(daoInterfaceName)) {
      xmlReaderContext.error("No DAO interface specified", element);
    }

    builder
        .addConstructorArgValue(daoInterfaceName)
        .addConstructorArgReference(ParserUtils.parseSessionFactory(element, xmlReaderContext))
        .addConstructorArgValue(element.getAttribute("entity"))
        .addConstructorArgValue(ParserUtils.getOrRegisterFinderAdvisor(xmlReaderContext));
  }

  /** {@inheritDoc} */
  @Override
  protected String getBeanClassName(final Element element) {
    return "net.sf.autodao.hibernate.HibernateDaoProxyFactoryBean";
  }
}
