/**
 * AutoDAO - Generic DAO on steroids implementation for Java.
 * Copyright (C) 2008-2009  Marat Radchenko <slonopotamus@users.sourceforge.net>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.autodao.hibernate;

import org.jetbrains.annotations.NotNull;
import org.springframework.beans.factory.config.BeanDefinition;
import static org.springframework.beans.factory.support.BeanDefinitionBuilder.genericBeanDefinition;
import org.springframework.beans.factory.support.BeanDefinitionRegistry;
import org.springframework.beans.factory.xml.XmlReaderContext;
import org.w3c.dom.Element;

/** Helper functions for custom Spring namespace tag parsing. */
final class ParserUtils {
  private static final String SESSION_FACTORY_ATTRIBUTE = "session-factory";

  static String parseSessionFactory(@NotNull final Element element, @NotNull final XmlReaderContext ctx) {
    if (element.hasAttribute(SESSION_FACTORY_ATTRIBUTE)) {
      return element.getAttribute(SESSION_FACTORY_ATTRIBUTE);
    } else {
      return "sessionFactory";
    }
  }

  @NotNull
  static String getOrRegisterFinderAdvisor(final XmlReaderContext context) {
    final String advisorClassName = "net.sf.autodao.FinderIntroductionAdvisor";
    final BeanDefinitionRegistry registry = context.getRegistry();
    for (final String beanName : registry.getBeanDefinitionNames()) {
      final BeanDefinition beanDefinition = registry.getBeanDefinition(beanName);
      if (advisorClassName.equals(beanDefinition.getBeanClassName())) {
        return beanName;
      }
    }
    return context.registerWithGeneratedName(genericBeanDefinition(advisorClassName).getBeanDefinition());
  }

  private ParserUtils() {
  }
}
